"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
/**
 * Read part of a file on disc.
 * @throws 'EPERM' if the file is not readable.
 */
async function readFile(filename, onlyFirstLine = false) {
    const chunkSize = 100;
    const stream = fs_1.default.createReadStream(filename, { highWaterMark: chunkSize });
    let content = '';
    for await (const data of stream) { // may throw
        content += data.toString();
        if (onlyFirstLine && content.includes('\n')) {
            return content.split(/\r?\n/)[0];
        }
    }
    return content;
}
exports.default = readFile;
